# gui.py
#
# A _really_ simple GUI library using pygame.

import colors as _colors
import pygame as _pygame
_pygame.init()

class GUIBox(object):
    """
    A "screen" or area for holding widgets.

    Note that a pygame subsurface is used. This means that changes
    automatically happen, so do not call blit() or whatever. Instead
    call update_surface().

    Probably not the best of designs. But whatever.
    """

    def __init__(self, screen, area, background = None):
        """
        screen - the display surface
        area - a pygame.Rect indicating area used for the GUIBox
        background - (optional) pygame.Surface used for background
        """
        area = _pygame.Rect(area)
        if background == None:
            background = _pygame.Surface(area[2:]).convert()
        self.__surface = screen.subsurface(area)
        self.__widgets = {}
        self.__background = background

    def __repr__(self):
        return "GUIBox(%s, %s, %s)" % (self.__surface,
                                       self.__surface.get_rect(),
                                       self.__background)
    
    def add(self, widget, position):
        """
        Add a widget to list. It is not an error to add a widget
        multiple times (nothing happens).

        widget - an instance of Widget (well, one of its subclasses)
        position - a two-element tuple (x, y) representing position
        on the GUIBox (not the display surface)
        """
        if widget not in self.__widgets:
            self.__widgets[widget] = position
            
    def remove(self, widget):
        """
        Remove a widget from list. It is not an error to remove a
        widget not in list.
        """
        if widget in self.__widgets:
            del self.__widgets[widget]
        
    def clear(self):
        """
        Remove all widgets from list.
        """
        self.__widgets.clear()

    def widgets_at_pos(self, position):
        """
        Returns a list of widgets at |position|.

        position - a two-element tuple (x, y) representing position
        on the display surface (not the GUIBox)
        """
        w = []
        for widget in self.__widgets:
            x = position[0] - self.__surface.get_offset()[0] - self.__widgets[widget][0]
            y = position[1] - self.__surface.get_offset()[0] - self.__widgets[widget][1]
            if widget.collidepoint((x, y)):
                w.append(widget)
        return w
    
    def update_surface(self):
        """
        Updates the surface.
        """
        self.__surface.blit(self.__background, (0, 0))
        for widget in self.__widgets:
            self.__surface.blit(widget.render(), self.__widgets[widget])

class Widget(object):
    """
    See definition of widget somewhere if needed.

    Besides overriding the abstract functions, all subclasses should
    have an attribute rect, indicating size of the widget. x and y
    should _always_ be (0, 0) - position is to be saved by the GUIBox,
    not the widget itself.
    """

    def __init__(self):
        raise NotImplementedError("Widget is an abstract base class")

    def __repr__(self):
        return "%s at <%s>" % (self.__class__.__name__, hex(id(self)))
    
    def collidepoint(self, position):
        """
        Returns True if position inside widget, False otherwise.
        For use by the GUIBox class, not client code.
        """
        return self.rect.collidepoint(position)

    def render(self):
        """
        Returns a pygame.Surface to be blit by the GUIBox.
        """
        raise NotImplementedError("Widget is an abstract base class")

class Label(Widget):
    """
    The most basic widget: plain text.
    """

    def __init__(self, text, size, color):
        """
        text - self-explanatory
        size - size of the font
        color - color of the font
        """
        self.text = text
        self.size = size
        self.color = color
        self.rect = self.render().get_rect()

    def __repr__(self):
        return "Label(%s, %s, %s)" % (self.text, self.size, self.color)

    def __str__(self):
        return self.text
    
    def render(self):
        f = _pygame.font.Font(None, self.size)
        f_im = f.render(self.text, True, self.color)
        return f_im

class Button(Widget):
    """
    The most famous example being the X normally present in the upper
    right corner of whatever window is open. (I think.)

    Uses an image, not text. Make an image containing text if you want text.
    Or something.
    """

    def __init__(self, im):
        self.image = im
        self.rect = self.render().get_rect()

    def __repr__(self):
        return "Button(%s)" % (self.__image)

    def render(self):
        return self.image

class BarGraph(Widget):
    """
    Usually used to compare things, like your level of fail (100%)
    versus my level of fail (0%).

    It's a horizontal bar graph, just for the record. Support for
    vertical ones are left for later (where later = infinity and beyond).
    """
    
    def __init__(self, title, width, text_size, **bars):
        """
        title - the title of the graph
        width - the width of the entire widget
        text_size - the size of the text used
        
        **bars - an arbitary number of keyword arguments.
        Taking an example, x = y.

        x - should describe that particular bar (e.g. fail)
        y - should describe pixels long (e.g. 100)

        Underscores are replaced by spaces.
        """
        self.title = title
        self.width = width
        self.text_size = text_size
        self.widget_color = _colors.white
        self.text_color = _colors.black
        self.bar_color = _colors.green
        self.bars = {}
        for key in bars:
            self.bars[key.replace("_", " ")] = bars[key]
        self.rect = self.render().get_rect()
        
    def update_bar(bar_name, new_value):
        """
        Updates |bar_name| to possess |new_value|.
        """
        self.bars[bar_name] = new_value

    def render(self):
        # create title and get some info
        title = Label(self.title, self.text_size, self.text_color)
        title_w, title_h = title.render().get_size()
        h = title_h + self.text_size + ((len(self.bars) - 1) * self.text_size)

        # get a list of label widths and use max label width to determine
        # x-offset from labels for the bars
        labels = [Label(bar, self.text_size, _colors.white) for bar in self.bars]
        label_widths = [l.render().get_width() for l in labels]
        max_label_width = max(label_widths)

        # create surface and blit bars to it
        surface = _pygame.Surface((self.width, h))
        surface.fill(self.widget_color)
        surface.blit(title.render(), (self.width / 2 - (title_w / 2), 0))
        y = self.text_size
        for bar_name in sorted(self.bars):
            surface.blit(Label(bar_name, self.text_size, self.text_color).render(),
                         (0, y))
            _pygame.draw.rect(surface, self.bar_color,
                              (max_label_width + 10, y, self.bars[bar_name], 10))
            y += self.text_size
            
        return surface
