'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import sys
MODE = "gui" # "gui" or "text"
            
import data
if __name__ == "__main__":
    print "Run using the run_game.py script one directory level up."
    raw_input()
    data.quit()

import yaml
import os, random
if MODE == "gui":
    import pygame
    from pygame.locals import *

    os.environ["SDL_VIDEO_CENTERED"] = "1"
    pygame.init()

import colors
import gui

if MODE == "gui":
    RESOLUTION = SCR_WIDTH, SCR_HEIGHT = 600, 600
    SCREEN = pygame.display.set_mode(RESOLUTION)
    pygame.display.set_caption("Project World of the Stars")
    pygame.mouse.set_cursor(*pygame.cursors.arrow)

# these classes should not define __repr__ by eval(repr(x)) == x
# because for any half-decent length of story chain, the printout
# will get really, really long and messy (each StoryNode maintains
# a list of StoryActions, which in turn each maintain a StoryNode
# they point to) and could even be infinite if the text file for
# some reason doesn't have some sort of ending

class StoryAction(object):
    def __init__(self, text, icon, next_node):
        self.text = text
        if MODE == "gui":
            try:
                if not icon:
                    raise pygame.error
                self.icon = data.load_image(icon)
            except(pygame.error):
                self.icon = data.load_image("next.bmp")
        else:
            self.icon = icon
        self.next_node = next_node

    def __repr__(self):
        return "StoryAction at <%s>" % (hex(id(self)))
    
class StoryNode(object):
    def __init__(self, node = "x", title = "Yet Another Day",
                 text = "What do you feel like doing?", *actions):
        self.node = node
        self.title = title
        self.text = text
        self.actions = list(actions)

    def __repr__(self):
        return "StoryNode at <%s>" % (hex(id(self)))

def word_wrap(s, max_width = 80):
    sections = s.split("\n")
    lines = []
    for section in sections:
        words = section.split(" ")
        current_line = []
        current_line_width = 0
        for w in words:
            current_line_width += len(w) + 1
            if current_line_width >= max_width:
                lines.append(" ".join(current_line))
                current_line = []
                current_line_width = len(w) + 1
            current_line.append(w)
        lines.append(" ".join(current_line))
        lines.append("")
        
    return lines
    
def init_story():
    game_nodes = {}
    raw_nodes = yaml.load(data.load("game_description.txt", "r"))
    for raw in raw_nodes:
        if raw["actions"]:
            actions = [StoryAction(**params) for params in raw["actions"]]
        else:
            actions = []
        game_nodes[raw["node"]] = StoryNode(raw["node"], raw["title"],
                                            raw["text"], *actions)
    return game_nodes



def text_game():
    print "*** Project World of the Stars ***"
    print
    game_nodes = init_story()
    current_node = game_nodes["start_node"]
    while True:
        print current_node.title
        print "+" + "-" * (len(current_node.title) - 2) + "+"
        print
        print "\n".join(word_wrap(current_node.text))
        print
        if not current_node.actions:
            raw_input("Press Enter to exit.")
            break
        for i, option in enumerate(current_node.actions):
            print "%d) %s" % (i + 1, option.text)
        while True:
            try:
                choice = int(raw_input()) - 1
                if choice < 0:
                    raise ValueError
                current_node = game_nodes[current_node.actions[choice].next_node]
            except(IndexError, ValueError):
                print "That was not a valid choice."
            else:
                os.system(['clear', 'cls'][os.name == 'nt'])
                break



def check_story_structure():
    game_nodes = init_story()
    for node in game_nodes:
        for action in game_nodes[node].actions:
            if action.next_node not in game_nodes:
                raise NameError("%s not found in %s" % (action.next_node,
                                                        node))
            
def update_gui(gui_box, current_node):
    """updates the GUI with new scenario (node) and returns the buttons"""
    gui_box.add(gui.Label(current_node.title, 30, colors.white), (10, 10))

    y = 50
    for line in word_wrap(current_node.text):
        gui_box.add(gui.Label(line, 20, colors.white), (10, y))
        y += 30

    buttons = []
    for i, option in enumerate(current_node.actions):
        buttons.append(gui.Button(option.icon))
        gui_box.add(buttons[i], (10, y))
        for j, line in enumerate(word_wrap(option.text, max_width = 70)):
            gui_box.add(gui.Label(line, 20, colors.white), (100, y + (j * 20)))
        y += 60
        
    return buttons


def fadeout_transition(old_screen, new_screen):
    old_screen_alpha = 255
    new_screen_alpha = 0
    d_alpha = 1
    while new_screen_alpha != 255:
        if old_screen_alpha >= d_alpha:
            old_screen_alpha -= d_alpha
        else:
            old_screen_alpha = 0

        if new_screen_alpha <= 255 - d_alpha:
            new_screen_alpha += d_alpha
        else:
            new_screen_alpha = 255
        old_screen.set_alpha(old_screen_alpha, RLEACCEL)
        new_screen.set_alpha(new_screen_alpha, RLEACCEL)
        SCREEN.fill(colors.black)
        SCREEN.blit(old_screen, (0, 0))
        SCREEN.blit(new_screen, (0, 0))
        pygame.display.update()

def slideright_transition(old_screen, new_screen):
    x = -SCR_WIDTH
    d_x = 1
    SCREEN.fill(colors.black)
    while x != 0:
        SCREEN.blit(new_screen, (x, 0))
        x += d_x
        if x > 0:
            x = 0
        pygame.display.update()
        
def gui_game():
    game_nodes = init_story()
    current_node = game_nodes["start_node"]
    box_surface = SCREEN.copy()
    box = gui.GUIBox(box_surface, (0, 0, SCR_WIDTH, SCR_HEIGHT))
    buttons = update_gui(box, current_node)
    transitions = (fadeout_transition, slideright_transition)
    
    while True:
        for event in pygame.event.get():
            if event.type == QUIT:
                return
            elif event.type == MOUSEBUTTONDOWN and event.button == 1:
                for i in range(len(buttons)):
                    if buttons[i] in box.widgets_at_pos(event.pos):
                        current_node = game_nodes[current_node.actions[i].next_node]
                        old_screen = box_surface.copy()
                        box.clear()
                        buttons = update_gui(box, current_node)
                        box.update_surface()
                        new_screen = box_surface.copy()
                        f = random.choice(transitions)
                        f(old_screen, new_screen)
                        break

        box.update_surface()
        SCREEN.blit(box_surface, (0, 0))
        pygame.display.update()

#main = globals()[MODE + "_game"]
main = gui_game
